#!/usr/bin/perl
use strict;

open my $log, '<', '/var/log/procwatch.log' or die "procwatch.log: $!\n";

my %h;

while (my $line = <$log>) {
	if (my ($uid, $gid, $flags, $rest) = $line =~ m{PID \d+ \(.*\) (\S+):(\S+) - \S+MB ram, .*cpu([^:]*): (.*)$}m) {
		if (my ($reason) = $rest =~ m{^killed for (\S+ \S+)}) {
			$h{$gid}->{kills}++;
			$h{$gid}->{ukills}->{$uid}++;
			$h{$gid}->{idle}->{$uid}++ if $flags =~ m{idle};
		} elsif ($rest =~ m{^reniced}) {
			$h{$gid}->{urenice}->{$uid}++;
			$h{$gid}->{renice}++;
		}
		$h{$gid}->{actions}++;
	}
}

for my $gid (reverse sort { $h{$a}->{actions} <=> $h{$b}->{actions} } keys %h) {
	if ($h{$gid}->{kills}) {
		my $count = $h{$gid}->{ukills};
		my $idle  = $h{$gid}->{idle};
		printf "Group %s killed %d times total (users: %s)\n", $gid, $h{$gid}->{kills}, join(" ", keys %$count);
		for my $uid (reverse sort { $count->{$a} <=> $count->{$b} } keys %$count) {
			printf "Group %s user %s killed %d times (%.1f%% idle PHP)\n", $gid, $uid, $count->{$uid}, 100 * $idle->{$uid} / $count->{$uid};
		}
	}

	if ($h{$gid}->{renice}) {
		my $count = $h{$gid}->{urenice};
		printf "Group %s reniced %d times total (users: %s)\n", $gid, $h{$gid}->{renice}, join(" ", keys %$count);
		for my $uid (reverse sort { $count->{$a} <=> $count->{$b} } keys %$count) {
			printf "Group %s user %s reniced %d times\n", $gid, $uid, $count->{$uid};
		}
	}

	print "\n";
}

exit 0;
